package cc.smtweb.system.bpm.web.sys.user.user;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.jdbc.AbsDbWorker;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.AbstractHandler;
import cc.smtweb.framework.core.util.CommUtil;
import cc.smtweb.framework.core.util.PubUtil;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageDelHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageListHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageLoadHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageSaveHandler;
import cc.smtweb.system.bpm.web.sys.user.role.RoleCache;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by Akmm at 2022/7/1 15:47
 * 区划保存
 */
public class UserHandler {

    public static EntityDao<User> getUserDao() {
        return DbEngine.getInstance().findDao(User.ENTITY_NAME);
    }

    public static EntityDao<UserStatu> getUserStatuDao() {
        return DbEngine.getInstance().findDao(UserStatu.ENTITY_NAME);
    }

    public static EntityDao<UserParty> getUserPartyDao() {
        return DbEngine.getInstance().findDao(UserParty.ENTITY_NAME);
    }

    public static EntityDao<UserRole> getUserRoleDao() {
        return DbEngine.getInstance().findDao(UserRole.ENTITY_NAME);
    }

    static class UserSaveHandler extends DynPageSaveHandler {
        private UserStatu userStatu;
        private List<UserRole> roleList;

        @Override
        protected DefaultEntity readBeanFromPage(PageDataset pageDataSet, SwMap data) {
            if (pageDataSet.isCurTable(UserParty.ENTITY_NAME)) {
                roleList = new ArrayList<>();
                Set<Long> userRoles = data.readLongSet("sup_roles");
                DefaultEntity bean = super.readBeanFromPage(pageDataSet, data);
                if (PubUtil.isEmpty(userRoles)) {
                    return bean;
                }
                userRoles.forEach(k -> {
                    UserRole userRole = new UserRole();
                    userRole.init();
                    userRole.setEntityId(DbEngine.getInstance().nextId());
                    userRole.setSupId(bean.getEntityId());
                    userRole.setRoleId(k);
                    roleList.add(userRole);
                });
                return bean;
            }
            return super.readBeanFromPage(pageDataSet, data);
        }

        @Override
        protected void saveBean(DefaultEntity bean) {
            super.saveBean(bean);
            if (bean instanceof User) {
                userStatu = UserStatuCache.getInstance().get(bean.getEntityId());
                if (userStatu == null || userStatu.isEmpty()) {
                    userStatu = new UserStatu();
                    userStatu.init();
                    userStatu.setEntityId(bean.getEntityId());
                    getUserStatuDao().insertEntity(userStatu);
                    UserStatuCache.getInstance().put(userStatu);
                }
            }
            if (bean instanceof UserParty) {
                Set<Long> ur = UserRoleCache.getInstance().getByParty(bean.getEntityId());
                getUserRoleDao().deleteEntity(ur);
                getUserRoleDao().batchInsertEntity(roleList);
                UserRoleCache.getInstance().removeList(ur);
                UserRoleCache.getInstance().putList(roleList);
            }
        }
    }

    static class UserDelHandler extends DynPageDelHandler {

        @Override
        protected void localDel(long id, ModelTable table) {
            super.localDel(id, table);
            if (table.getName().equals(User.ENTITY_NAME)) {
                getUserStatuDao().deleteEntity(id);
                getUserPartyDao().deleteEntity(UserPartyCache.getInstance().getIdByUser(id));
                getUserRoleDao().deleteEntity(UserRoleCache.getInstance().getIdByUser(id));
            }
            if (table.getName().equals(UserParty.ENTITY_NAME)) {
                getUserPartyDao().deleteEntity(id);
                getUserRoleDao().deleteEntity(UserRoleCache.getInstance().getByParty(id));
            }
        }

        @Override
        protected void localDelSuccess(long id, ModelTable table) {
            if (table.getName().equals(User.ENTITY_NAME)) {
                UserStatuCache.getInstance().remove(id);
                UserPartyCache.getInstance().removeList(UserPartyCache.getInstance().getIdByUser(id));
                UserRoleCache.getInstance().removeList(UserRoleCache.getInstance().getIdByUser(id));
            }
            if (table.getName().equals(UserParty.ENTITY_NAME)) {
                UserPartyCache.getInstance().remove(id);
                UserRoleCache.getInstance().removeList(UserRoleCache.getInstance().getByParty(id));
            }
        }
    }

    static class UserListHandler extends DynPageListHandler {

        public UserListHandler(long pageId, SwMap filter, PageDataset pageDataSet) {
            super(pageId, filter, pageDataSet);
        }

        @Override
        protected void afterQuery(List<SwMap> listData) {
            if (pageDataSet.isCurTable(UserParty.ENTITY_NAME)) {
                listData.forEach(sw -> {
                    sw.put("sup_roles", RoleCache.getInstance().getNamesByIds(UserRoleCache.getInstance().getRoleByParty(sw.readLong("sup_id"))));
                });
                return;
            }
            super.afterQuery(listData);
        }
    }

    static class UserLoadHandler extends DynPageLoadHandler {

        @Override
        public DynPageListHandler getListWorker(SwMap filter, PageDataset pageDataSet) {
            UserListHandler listHandler = new UserListHandler(pageId, filter, pageDataSet);
            listHandler.init(params, us);
            return listHandler;
        }


        @Override
        protected void afterLoadBean(PageDataset dataset, SwMap bean) {
            super.afterLoadBean(dataset, bean);
            if (dataset.isCurTable(UserParty.ENTITY_NAME)) {
                bean.put("sup_roles", CommUtil.getSqlInIds(UserRoleCache.getInstance().getRoleByParty(bean.readLong("sup_id"))));
            }
        }
    }

    static class UserCustHandler extends AbstractHandler {

        public R batchDelParty() {
            Set<Long> idList = params.readLongSet("ids");
            Set<Long> roleIdList = UserRoleCache.getInstance().getByParty(idList);
            DbEngine.getInstance().doTrans(new AbsDbWorker() {
                @Override
                public void work() {
                    getUserPartyDao().deleteEntity(idList);
                    getUserRoleDao().deleteEntity(roleIdList);
                }

                @Override
                public void doAfterDbCommit() {
                    UserPartyCache.getInstance().removeList(idList);
                    UserRoleCache.getInstance().removeList(roleIdList);
                }
            });
            return R.success();
        }

        public R delAll() {
            long id = params.readLong("id");
            Set<Long> partyIds = UserPartyCache.getInstance().getIdByUser(id);
            Set<Long> roleIds = UserRoleCache.getInstance().getByParty(partyIds);
            DbEngine.getInstance().doTrans(new AbsDbWorker() {
                @Override
                public void work() {
                    getUserDao().deleteEntity(id);
                    getUserStatuDao().deleteEntity(id);
                    getUserRoleDao().deleteEntity(roleIds);
                    getUserPartyDao().deleteEntity(partyIds);
                }

                @Override
                public void doAfterDbCommit() {
                    UserCache.getInstance().remove(id);
                    UserStatuCache.getInstance().remove(id);
                    UserRoleCache.getInstance().removeList(roleIds);
                    UserPartyCache.getInstance().removeList(partyIds);
                }
            });
            return R.success();
        }
    }
}
